from multiprocessing import cpu_count


ENCODING_OPTIONS = {
    'mp3': '-c:a libmp3lame '
           '-q:a 3 ',
    'h264': '-c:v libx264 '
            '-preset ultrafast '
            '-crf 21 ',
    'mp4': '-f mp4'
}

COPY_OPTIONS = {
    'audio': '-c:a copy',
    'video': '-c:v copy',
    'container': ''
}

THREADS = cpu_count()

AUDIO_CODEC = 'mp3'
VIDEO_CODEC = 'h264'
CONTAINER_TYPE = 'mp4'

NEW_FILE_FMT = '%s_castconvert.mp4'


from concurrent.futures import ThreadPoolExecutor as TPE
from queue import Queue
from time import sleep, time

from os.path import getsize
from watchdog.events import PatternMatchingEventHandler
from watchdog.observers import Observer

from .media_info import is_video
from .convert import convert_video


SIZE_CHECK_WAIT = 1


def file_size_stable(filename: str, wait: float = SIZE_CHECK_WAIT, previous: int = -1):
    while True:
        sleep(wait)

        filesize = getsize(filename)

        if filesize == previous:
            return

        previous = filesize


def consume_video_queue(queue: Queue):
    seen = set()

    while True:
        filename = queue.get()

        if filename in seen:
            continue

        file_size_stable(filename)

        if is_video(filename):
            print(convert_video(filename))

        else:
            print(filename, 'not video')

        seen.add(filename)


class AddedFileHandler(PatternMatchingEventHandler):
    def __init__(self, queue: Queue, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.queue = queue

    def on_modified(self, event):
        if event.is_directory:
            return

        self.queue.put(event.src_path)

    def on_moved(self, event):
        self.on_modified(event)

    def on_created(self, event):
        self.on_modified(event)


def watch_directory(directory: str, ignore_patterns: list=[]):
    observer = Observer()
    queue = Queue()
    handler = AddedFileHandler(queue, ignore_patterns=ignore_patterns)

    thread_pool = TPE(1)
    converter_future = thread_pool.submit(consume_video_queue, queue)

    print("Watching %s for new videos..." % directory)

    observer.schedule(handler, directory, recursive=True)
    observer.start()

    try:
        while True:
            sleep(1)

    except KeyboardInterrupt:
        observer.stop()

    try:
        observer.join()
        queue.join()
        converter_future.cancel()
        thread_pool.shutdown(wait=0.1)

    except KeyboardInterrupt:
        exit()




from setuptools import setup
from sys import version_info
from cast_convert import __version__


MIN_PYTHON_VERSION = (3, 5)
OLD_PYTHON_REQUIREMENTS = ['mypy-lang']

with open('requirements.txt', 'r') as file:
    requirements = file.readlines()

if version_info < MIN_PYTHON_VERSION:
    requirements.extend(OLD_PYTHON_REQUIREMENTS)

setup(name="cast_convert",
      version=__version__,
      description="Convert and inspect video for Chromecast playback",
      url="https://github.com/thismachinechills/cast_convert",
      author="thismachinechills (Alex)",
      license="AGPL 3.0",
      packages=['cast_convert'],
      zip_safe=True,
      install_requires=requirements,
      keywords=["chromecast", "ffmpeg", "transcode"],
      entry_points={"console_scripts": ["cast_convert = cast_convert:command"]})
